#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2023/12/08 10:14:40
@Author  :   ChenHao
@Contact :   jerrychen1990@gmail.com
'''

from agit.common import *
from xagents.agent.xagent import XAgent